/*
 * vim: ts=4 nobackup
 */


#include <stdint.h>
#include <avr/io.h>
#include "I2C.H"


using namespace avelino;
using namespace std;


#define  I2C_STATUS_IDLE             0
#define  I2C_STATUS_WRITE_SEQUENCE   1
#define  I2C_STATUS_READ_SEQUENCE_1  2
#define  I2C_STATUS_READ_SEQUENCE_2  3
#define  I2C_STATUS_READ_SEQUENCE_3  4


uint8_t I2C::status;
uint8_t I2C::address;
uint8_t *I2C::buffer;
uint16_t I2C::bufferIndex;
uint16_t I2C::bufferSize;
bool I2C::sendStopAtEnd;


void I2C::init() {
	TWSR = 0x00;
	// TWBR = 12;   // 400KHz
	TWBR = 72;    // 100KHz
	TWCR = (1 << TWEN);
	status = I2C_STATUS_IDLE;
}


void I2C::start() {
	TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN);
}


void I2C::stop() {
	TWCR = (1 << TWINT) | (1 << TWSTO) | (1 << TWEN);
}


void I2C::write(uint8_t v) {
	TWDR = v;
	TWCR = (1 << TWINT) | (1 << TWEN);
}


void I2C::readACK() {
	TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWEA);
}


void I2C::readNACK() {
	TWCR = (1 << TWINT) | (1 << TWEN);
}


uint8_t I2C::getReaded() {
	return TWDR;
}


bool I2C::isReady() {
	return ((TWCR & (1 << TWINT)) != 0);
}


void I2C::writeSequence(uint8_t *data, uint16_t size, bool sendStop) {
	buffer = data;
	bufferIndex = 0;
	bufferSize = size;
	sendStopAtEnd = sendStop;
	start();
	status = I2C_STATUS_WRITE_SEQUENCE;
}


void I2C::readSequence(uint8_t addr, uint8_t *data, uint16_t size, bool sendStop) {
	address = addr;
	buffer = data;
	bufferIndex = 0;
	bufferSize = size;
	sendStopAtEnd = sendStop;
	start();
	status = I2C_STATUS_READ_SEQUENCE_1;
}


bool I2C::sequenceDone() {
	return (status == I2C_STATUS_IDLE);
}


void I2C::run() {
	if (status == I2C_STATUS_WRITE_SEQUENCE) {
		if (isReady()) {
			if (bufferIndex == bufferSize) {
				if (sendStopAtEnd)
					stop();
				status = I2C_STATUS_IDLE;
			}
			else {
				write(buffer[bufferIndex]);
				bufferIndex++;
			}
		}
	}
	else if (status == I2C_STATUS_READ_SEQUENCE_1) {
		if (isReady()) {
			write(address);
			status = I2C_STATUS_READ_SEQUENCE_2;
		}
	}
	else if (status == I2C_STATUS_READ_SEQUENCE_2) {
		if (isReady()) {
			if (bufferIndex == (bufferSize - 1))
				readNACK();
			else
				readACK();
			status = I2C_STATUS_READ_SEQUENCE_3;
		}
	}
	else if (status == I2C_STATUS_READ_SEQUENCE_3) {
		if (isReady()) {
			buffer[bufferIndex] = getReaded();
			bufferIndex++;
			if (bufferIndex == bufferSize) {
				if (sendStopAtEnd)
					stop();
				status = I2C_STATUS_IDLE;
			}
			else {
				if (bufferIndex == (bufferSize - 1))
					readNACK();
				else
					readACK();
			}
		}
	}
}

