/*
 * vim: ts=4 nobackup
 */


#include <stdint.h>
#include "I2C.H"
#include "RTC.H"


using namespace avelino;
using namespace std;


#define  RTC_STATUS_IDLE             0
#define  RTC_STATUS_READ_HALT_BIT_1  1
#define  RTC_STATUS_READ_HALT_BIT_2  2
#define  RTC_STATUS_READ_HALT_BIT_3  3
#define  RTC_STATUS_READ_HMS_1       4
#define  RTC_STATUS_READ_HMS_2       5
#define  RTC_STATUS_WRITE_HMS_1      6


uint8_t RTC::status;
uint8_t RTC::aux[4];
uint8_t RTC::haltByte;
bool RTC::hourMinuteSecondReadRequested;
bool RTC::hourMinuteSecondReadIsOk;
bool RTC::hourMinuteSecondWriteRequested;
bool RTC::hourMinuteSecondWriteIsOk;
uint8_t RTC::readedHour;
uint8_t RTC::readedMinute;
uint8_t RTC::readedSecond;
uint8_t RTC::hourToWrite;
uint8_t RTC::minuteToWrite;
uint8_t RTC::secondToWrite;


void RTC::init() {
	I2C::init();
	aux[0] = 0xD0;
	aux[1] = 0x00;
	I2C::writeSequence(aux, 2, false);
	status = RTC_STATUS_READ_HALT_BIT_1;
}


void RTC::requestHourMinuteSecondRead() {
	hourMinuteSecondReadRequested = true;
	hourMinuteSecondReadIsOk = false;
}


bool RTC::isHourMinuteSecondReadOk() {
	return hourMinuteSecondReadIsOk;
}


bool RTC::isHourMinuteSecondReadRequested() {
	return hourMinuteSecondReadRequested;
}


void RTC::requestHourMinuteSecondWrite(uint8_t h, uint8_t m, uint8_t s) {
	hourToWrite = h;
	minuteToWrite = m;
	secondToWrite = s;
	hourMinuteSecondWriteRequested = true;
	hourMinuteSecondWriteIsOk = false;
}


bool RTC::isHourMinuteSecondWriteOk() {
	return hourMinuteSecondWriteIsOk;
}


bool RTC::isHourMinuteSecondWriteRequested() {
	return hourMinuteSecondWriteRequested;
}


void RTC::run() {
	I2C::run();
	if (status == RTC_STATUS_READ_HALT_BIT_1) {
		if (I2C::sequenceDone()) {
			I2C::readSequence(0xD1, &haltByte, 1, true);
			status = RTC_STATUS_READ_HALT_BIT_2;
		}
	}
	else if (status == RTC_STATUS_READ_HALT_BIT_2) {
		if (I2C::sequenceDone()) {
			if ((haltByte & 0x80) != 0) {
				aux[0] = 0xD0;
				aux[1] = 0x00;
				aux[2] = haltByte & 0x7F;
				I2C::writeSequence(aux, 3, true);
				status = RTC_STATUS_READ_HALT_BIT_3;
			}
			else
				status = RTC_STATUS_IDLE;
		}
	}
	else if (status == RTC_STATUS_READ_HALT_BIT_3) {
		if (I2C::sequenceDone()) {
			status = RTC_STATUS_IDLE;
		}
	}
	else if (status == RTC_STATUS_IDLE) {
		if (hourMinuteSecondReadRequested) {
			aux[0] = 0xD0;
			aux[1] = 0x00;
			I2C::writeSequence(aux, 2, false);
			status = RTC_STATUS_READ_HMS_1;
		}
		else if (hourMinuteSecondWriteRequested) {
			aux[0] = 0xD0;
			aux[1] = 0x00;
			aux[2] = secondToWrite;
			aux[3] = minuteToWrite;
			aux[4] = hourToWrite;
			I2C::writeSequence(aux, 5, true);
			status = RTC_STATUS_WRITE_HMS_1;
		}
	}
	else if (status == RTC_STATUS_READ_HMS_1) {
		if (I2C::sequenceDone()) {
			I2C::readSequence(0xD1, aux, 3, true);
			status = RTC_STATUS_READ_HMS_2;
		}
	}
	else if (status == RTC_STATUS_READ_HMS_2) {
		if (I2C::sequenceDone()) {
			readedSecond = aux[0];
			readedMinute = aux[1];
			readedHour = aux[2];
			hourMinuteSecondReadIsOk = true;
			hourMinuteSecondReadRequested = false;
			status = RTC_STATUS_IDLE;
		}
	}
	else if (status == RTC_STATUS_WRITE_HMS_1) {
		if (I2C::sequenceDone()) {
			hourMinuteSecondWriteIsOk = true;
			hourMinuteSecondWriteRequested = false;
			status = RTC_STATUS_IDLE;
		}
	}
}

