/*
 * vim: ts=4 nobackup
 */


#include <stdint.h>
#include <avr/io.h>
#include "Timer.H"
#include "RTC.H"
#include "RTCResetButton.H"


using namespace avelino;
using namespace std;


int8_t RTCResetButton::timerHandler;
uint8_t RTCResetButton::status;
RTCResetButtonListener *RTCResetButton::listener;


#define  DELAY_MS                 2000
#define  TIMER_COUNTER_FOR_DELAY  TIMER_COUNT_FROM_MS(DELAY_MS)


#define  RTC_RESET_BUTTON_STATUS_IDLE           0
#define  RTC_RESET_BUTTON_STATUS_WAIT_PRESSED   1
#define  RTC_RESET_BUTTON_STATUS_WAIT_RELEASED  2


void RTCResetButton::init(RTCResetButtonListener &l) {
	DDRB &= 0xEF;   // button at PORTB.4
	listener = &l;
	timerHandler = Timer::alloc();
	status = RTC_RESET_BUTTON_STATUS_IDLE;
}


void RTCResetButton::run() {
	if (status == RTC_RESET_BUTTON_STATUS_IDLE) {
		if ((PINB & 0x10) == 0) {   // pressed when go to 0 volts
			Timer::counter[timerHandler] = TIMER_COUNTER_FOR_DELAY;
			status = RTC_RESET_BUTTON_STATUS_WAIT_PRESSED;
		}
	}
	else if (status == RTC_RESET_BUTTON_STATUS_WAIT_PRESSED) {
		if (Timer::counter[timerHandler] == 0) {
			if ((PINB & 0x10) == 0) {
				// button remains pressed after 2s
				// write 0:00:02, because the button delay is 2 second
				RTC::requestHourMinuteSecondWrite(0x00, 0x00, 0x02);
				listener->RTCResetButtonPressed();
				status = RTC_RESET_BUTTON_STATUS_WAIT_RELEASED;
				// wait again to avoid spurious bounces
				Timer::counter[timerHandler] = TIMER_COUNTER_FOR_DELAY;
			}
			else {
				// its a "fake" pressing
				status = RTC_RESET_BUTTON_STATUS_IDLE;
			}
		}
	}
	else if (status == RTC_RESET_BUTTON_STATUS_WAIT_RELEASED) {
		if (Timer::counter[timerHandler] == 0)
			status = RTC_RESET_BUTTON_STATUS_IDLE;
	}
}

