/*
 * vim: ts=4 nobackup
 */


#include <stdint.h>
#include <avr/io.h>
#include "Timer.H"
#include "RTCResetLed.H"


using namespace avelino;
using namespace std;


#define  DELAY_ON_MS            2000
#define  TIMER_COUNT_DELAY_ON   TIMER_COUNT_FROM_MS(DELAY_ON_MS)
#define  DELAY_OFF_MS           2000
#define  TIMER_COUNT_DELAY_OFF  TIMER_COUNT_FROM_MS(DELAY_OFF_MS)


#define  RTC_RESET_LED_STATUS_IDLE      0
#define  RTC_RESET_LED_STATUS_WAIT_ON   1
#define  RTC_RESET_LED_STATUS_WAIT_OFF  2


void RTCResetLed::init() {
	this->timerHandler = Timer::alloc();
	this->status = RTC_RESET_LED_STATUS_IDLE;
	this->buttonPressed = false;
	DDRE |= 0x40;    // reset led at PORTE.6
}


void RTCResetLed::run() {
	if (this->status == RTC_RESET_LED_STATUS_IDLE) {
		if (this->buttonPressed) {
			PORTE |= 0x40;   // on
			this->buttonPressed = false;
			Timer::counter[this->timerHandler] = TIMER_COUNT_DELAY_ON;
			this->status = RTC_RESET_LED_STATUS_WAIT_ON;
		}
	}
	else if (this->status == RTC_RESET_LED_STATUS_WAIT_ON) {
		if (Timer::counter[this->timerHandler] == 0) {
			PORTE &= 0xBF;   // off
			Timer::counter[this->timerHandler] = TIMER_COUNT_DELAY_OFF;
			this->status = RTC_RESET_LED_STATUS_WAIT_OFF;
		}
	}
	else if (this->status == RTC_RESET_LED_STATUS_WAIT_OFF) {
		if (Timer::counter[this->timerHandler] == 0)
			this->status = RTC_RESET_LED_STATUS_IDLE;
	}
}


void RTCResetLed::RTCResetButtonPressed() {
	this->buttonPressed = true;
}

