/*
 * vim: ts=4 nobackup
 */


#include <stdint.h>
#include <stdlib.h>
#include "Timer.H"
#include "ADCManager.H"
#include "StarsManager.H"


using namespace avelino;
using namespace std;


#define  DELAY_MS                 1000
#define  TIMER_COUNTER_FOR_DELAY  TIMER_COUNT_FROM_MS(DELAY_MS)


#define  DAY_PROB_PERCENT    10
#define  NIGHT_PROB_PERCENT  80


#define  DAY_START_BCD_HOUR    0x07
#define  NIGHT_START_BCD_HOUR  0x20




void StarsManager::init(LedGroup &lg) {
	ADCManager::init();
	srand(ADCManager::get(0));
	this->ledGroup = &lg;
	uint8_t n = this->ledGroup->getNumLeds();
	for (uint8_t i = 0; i < n; i++)
		this->ledGroup->setLed(i, true);
	this->timerHandler = Timer::alloc();
	this->probPercent = 50;
}


void StarsManager::setProbPercent(uint8_t v) {
	this->probPercent = v;
}


void StarsManager::run() {
	if (Timer::counter[this->timerHandler] > 0)
		return;
	uint8_t n = this->ledGroup->getNumLeds();
	for (uint8_t i = 0; i < n; i++) {
		uint8_t r = rand() % 100;   // 0 ... 99
		if (r < this->probPercent)
			this->ledGroup->setLed(i, true);
		else
			this->ledGroup->setLed(i, false);
	}
	Timer::counter[this->timerHandler] = TIMER_COUNTER_FOR_DELAY;
}


void StarsManager::RTCChanged(uint8_t hour, uint8_t minute, uint8_t second) {
	if ((hour >= DAY_START_BCD_HOUR) && (hour < NIGHT_START_BCD_HOUR))
		this->probPercent = DAY_PROB_PERCENT;
	else
		this->probPercent = NIGHT_PROB_PERCENT;
}

